# Nasdaq Data Link IMF Cross Country Macroeconomic Statistics dataset script
# The script uses a prepared file that should be changed according to the series required
# Information on series and codes is available through the available documentation online:
# https://data.nasdaq.com/data/ODA-imf-cross-country-macroeconomic-statistics/documentation
# Author: Mark Bruyneel
# Date: February 12th, 2022
#

# Import necessary python modules
from csv import reader
import nasdaqdatalink
from AK_config import AK
import pandas as pd
import numpy as np
import matplotlib.pyplot

# Use Authentication key
nasdaqdatalink.ApiConfig.api_key = AK

# Now import the list of IMF series
with open('U:\Werk\Data Management\Python\Files\quandl_tests\IMF_series.txt', "r") as imfseries_i:
    imfserieslist = reader(imfseries_i)
    imfseries = list(imfserieslist)
    imfseriesnames = imfseries[0].copy()

# Create a new list of names for Long conversion of series
positions = []
for item in imfseriesnames:
    imfvar = item[4:]+"_"+item[0:3]
    positions.append(imfvar)

# Create a new list of variables to use to get the raw data
Seriesnr = len(imfseries[0])
IMFserieslist = ['IMF'] * Seriesnr
imfnrList = list(range(1, (Seriesnr+1)))
IMFNrList = map(str, imfnrList)
IMFall = [i + j for i, j in zip(IMFserieslist, IMFNrList)]

# Create a second new list of variables to use to put a copy of part of the data in
Seriesnr = len(imfseries[0])
IMFCserieslist = ['IMFNS'] * Seriesnr
imfnrList = list(range(1, (Seriesnr+1)))
IMFNrList = map(str, imfnrList)
IMF_series = [i + j for i, j in zip(IMFCserieslist, IMFNrList)]

# Show a graph with all IMF data or just a range of years and save the data as a csv file
answer = None
while answer not in ("yes", "no"):
    answer = input('Do you want a specific range of years to compare them (yes/no) ?')
    if answer == "yes":
        startyear = input('What year do you want start (YYYY) ?\n')
        endyear = input('What year do you want to end (YYYY) ?\n')

        # Getting the data from Nasdaq Data Link and copying only the series data
        IMF_series_to_combine = []
        for origlist, origimfdat, cpy_imfdat in zip(imfseries[0], IMFall, IMF_series):
            origimfdat = nasdaqdatalink.get('ODA/' + origlist, start_date = startyear+'-01-01', end_date = endyear+'-12-31')
            cpy_imfdat = origimfdat.copy()[['Value']]
            # rename column headers for the graph
            cpy_imfdat.rename(columns={'Value': origlist}, inplace=True)
            IMF_series_to_combine.append(cpy_imfdat)
    elif answer == "no":
        # Getting the data from Nasdaq Data Link and copying only the series data
        IMF_series_to_combine = []
        for origlist, origimfdat, cpy_imfdat in zip(imfseries[0], IMFall, IMF_series):
            origimfdat = nasdaqdatalink.get('ODA/' + origlist)
            cpy_imfdat = origimfdat.copy()[['Value']]
            # rename column headers for the graph
            cpy_imfdat.rename(columns={'Value': origlist}, inplace=True)
            IMF_series_to_combine.append(cpy_imfdat)
    else:
        print("Please enter yes or no.")

# Join all stocks together in table
# Start by telling that the new table should start with the first series [0]
allIMFseries = IMF_series_to_combine[0]
# Now merge the other series from the list Stocks_to_combine starting with the second
for IMFlists in IMF_series_to_combine[1:]:
    allIMFseries = allIMFseries.merge(IMFlists, how='inner', left_index=True, right_index=True)

#allIMFseries.plot()
df = pd.DataFrame(allIMFseries)
df.to_csv(r'U:\Werk\Data Management\Python\Files\output\IMFseries.csv', header=True)

# Create a new variable with the date as a copy of the index (= not a column)
allIMFseries['Date'] = allIMFseries.index
allIMFseries['year'] = pd.DatetimeIndex(allIMFseries['Date']).year

# create a list of our conditions
conditions = [
    (allIMFseries['year'] <= 1989),
    (allIMFseries['year'] > 1989) & (allIMFseries['year'] <= 1999),
    (allIMFseries['year'] > 1999) & (allIMFseries['year'] <= 2009),
    (allIMFseries['year'] > 2009) & (allIMFseries['year'] <= 2019),
    (allIMFseries['year'] > 2019)
    ]
# Create a list of categories
categories = ['80s', '90s', '2000s', '2010s', '2020s']
cat_nr = [1, 2, 3, 4, 5]

# Create a new variable bracket and input the correct bracket
allIMFseries['bracket'] = np.select(conditions, categories)
allIMFseries['cat_nr'] = np.select(conditions, cat_nr)
allIMFseries.columns = positions + ['Date', 'year', 'bracket', 'cat_nr']

# Group data by bracket and type of item
Grouped_allIMFseries = allIMFseries.groupby(['bracket']).mean()
Grouped_allIMFseries.sort_values(by=['cat_nr'], inplace=True)
print(Grouped_allIMFseries)

# Plot in bar graph only data columns
Grouped_allIMFseries[positions].plot.bar()
matplotlib.pyplot.show()